import pprint
import datetime
import json

import pydgraph



# Create a client stub.
def create_client_stub():
    return pydgraph.DgraphClientStub('localhost:9080')


# Create a client.
def create_client(client_stub):
    return pydgraph.DgraphClient(client_stub)



client_stub = create_client_stub()
client = create_client(client_stub)



# Drop All - discard all data and start from a clean slate.
def drop_all(client):
    return client.alter(pydgraph.Operation(drop_all=True))


# Set schema.
def set_schema(client):
    schema = """

    type Person {
        name: string
        age: int
        friend: [Person]
    }

    type Animal {
        name: string
    }

    # Define Directives and index

    name: string @index(term) @lang .
    age: int @index(int) .
    friend: [uid] @count .
    """
    return client.alter(pydgraph.Operation(schema=schema))



def create_data_from_tour(client):
    txn = client.txn()

    nquads = """
    _:michael <name> "Michael" .
    _:michael <dgraph.type> "Person" .
    _:michael <age> "39" .
    _:michael <friend> _:amit .
    _:michael <friend> _:sarah .
    _:michael <friend> _:sang .
    _:michael <friend> _:catalina .
    _:michael <friend> _:artyom .
    _:michael <owns_pet> _:rammy .

    _:amit <name> "अमित"@hi .
    _:amit <name> "অমিত"@bn .
    _:amit <name> "Amit"@en .
    _:amit <dgraph.type> "Person" .
    _:amit <age> "35" .
    _:amit <friend> _:michael .
    _:amit <friend> _:sang .
    _:amit <friend> _:artyom .

    _:luke <name> "Luke"@en .
    _:luke <dgraph.type> "Person" .
    _:luke <name> "Łukasz"@pl .
    _:luke <age> "77" .

    _:artyom <name> "Артём"@ru .
    _:artyom <name> "Artyom"@en .
    _:artyom <dgraph.type> "Person" .
    _:artyom <age> "35" .

    _:sarah <name> "Sarah" .
    _:sarah <dgraph.type> "Person" .
    _:sarah <age> "55" .

    _:sang <name> "상현"@ko .
    _:sang <name> "Sang Hyun"@en .
    _:sang <dgraph.type> "Person" .
    _:sang <age> "24" .
    _:sang <friend> _:amit .
    _:sang <friend> _:catalina .
    _:sang <friend> _:hyung .
    _:sang <owns_pet> _:goldie .

    _:hyung <name> "형신"@ko .
    _:hyung <name> "Hyung Sin"@en .
    _:hyung <dgraph.type> "Person" .
    _:hyung <friend> _:sang .

    _:catalina <name> "Catalina" .
    _:catalina <dgraph.type> "Person" .
    _:catalina <age> "19" .
    _:catalina <friend> _:sang .
    _:catalina <owns_pet> _:perro .

    _:rammy <name> "Rammy the sheep" .
    _:rammy <dgraph.type> "Animal" .

    _:goldie <name> "Goldie" .
    _:goldie <dgraph.type> "Animal" .

    _:perro <name> "Perro" .
    _:goldie <dgraph.type> "Animal" .
    """

    try:
        assigned = txn.mutate(set_nquads = nquads)
        # Commit transaction.
        txn.commit()

        # Get uid of the outermost object (person named "Alice").
        # assigned.uids returns a map from blank node names to uids.
        # For a json mutation, blank node names "blank-0", "blank-1", ... are used
        # for all the created nodes.
        print('Created person named "Alice" with uid = {}\n'.format(assigned.uids['goldie']))

        print('All created nodes (map from blank node names to uids):')
        for uid in assigned.uids:
            print('{} => {}'.format(uid, assigned.uids[uid]))
    finally:
        # Clean up. Calling this after txn.commit() is a no-op
        # and hence safe.
        txn.discard()
        print('\n')


# Create data using JSON.
def create_data(client):
    # Create a new transaction.
    txn = client.txn()
    try:
        # Create data.
        p = {
            'name': 'Alice',
            'age': 26,
            'married': True,
            'loc': {
                'type': 'Point',
                'coordinates': [1.1, 2],
            },
            'dob': datetime.datetime(1980, 1, 1, 23, 0, 0, 0).isoformat(),
            'friend': [
                {
                    'name': 'Bob',
                    'age': 24,
                },
                {
                    'name': 'Charlie',
                    'age': 29,
                }
            ],
            'school': [
                {
                    'name': 'Crown Public School',
                }
            ]
        }

        # Run mutation.
        assigned = txn.mutate(set_obj=p)

        # Commit transaction.
        txn.commit()

        # Get uid of the outermost object (person named "Alice").
        # assigned.uids returns a map from blank node names to uids.
        # For a json mutation, blank node names "blank-0", "blank-1", ... are used
        # for all the created nodes.
        print('Created person named "Alice" with uid = {}\n'.format(assigned.uids['blank-0']))

        print('All created nodes (map from blank node names to uids):')
        for uid in assigned.uids:
            print('{} => {}'.format(uid, assigned.uids[uid]))
    finally:
        # Clean up. Calling this after txn.commit() is a no-op
        # and hence safe.
        txn.discard()
        print('\n')


#Deleting a data
def delete_data(client):
    # Create a new transaction.
    txn = client.txn()
    try:
        query1 = """query all($a: string)
        {
           all(func: eq(name, $a))
            {
               uid
            }
        }"""
        variables1 = {'$a': 'Bob'}
        res1 = client.txn(read_only=True).query(query1, variables=variables1)
        ppl1 = json.loads(res1.json)
        for person in ppl1['all']:
            print('Query to find Uid for Bob :')
            print(query1)
            print('\n')
            print("Bob's UID : ")
            print(person)
            print('\n')
            print('Bob deleted')
            print('\n')
        assigned = txn.mutate(del_obj= person)

        txn.commit()

    finally:
        txn.discard()


# Query for data.
def query_data(client):
    # Run query.
    query = """query all($a: string) {
        all(func: eq(name, $a)) {
            uid
            name@en:.
            age
            friend {
                name@en:.
                age
            }
        }
    }"""

    variables = {'$a': 'Michael'}
    res = client.txn(read_only=True).query(query, variables=variables)
    ppl = json.loads(res.json)

    # Print results.
    print('Number of people named "Michael": {}'.format(len(ppl['all'])))
    print('\n')
    for person in ppl['all']:
        print('Query for Michael : \n' +query)
        print('\n')
        print('Result :')
        print(person)
        print('\n')



#Query to check for deleted node
def query_data01(client):
    query01 = """query all($b: string)
        {   all(func: eq(name, $b))
            {   uid,
                name,
                age
                friend
                {
                    uid,
                    name,
                    age
                }
                ~friend
                {
                    uid,
                    name,
                    age
                }
            }
        }"""

    variables01 = {'$b': 'Bob'}
    res01 = client.txn(read_only=True).query(query01, variables=variables01)
    ppl01 = json.loads(res01.json)

    print('Number of people named "Bob": {}'.format(len(ppl01['all'])))
    print('\n')
    for person in ppl01['all']:
        print('Query for Bob :\n' + query01)
        print('\n')
        print('Result :')
        print(person)
        print('\n')


def close_client(client_stub):
    client_stub.close()


def main():
    client_stub = create_client_stub()
    client = create_client(client_stub)
    drop_all(client)
    set_schema(client)
    create_data(client)
    query_data(client) # query for Alice
    query_data01(client) # query for Bob
    delete_data(client) # delete Bob
    query_data(client) # query for Alice
    query_data01(client) # query for Bob

    # Close the client stub.
    client_stub.close()



# No need to expose the client to the consumer of this library
def get_schema():
    query01 = """schema {}"""

    res01 = client.txn(read_only=True).query(query01)
    schema = json.loads(res01.json)

    # types = schema['types']
    # edges = schema['schema']

    return schema

def get_node_page(node_type):
    query01 = """query all($node_type: string)
        {   all(func: type($node_type), first: 50)
            { uid
              expand(_all_) { uid }
            }
        }"""

    variables01 = {'$node_type': node_type}
    res01 = client.txn(read_only=True).query(query01, variables=variables01)
    node_examples = json.loads(res01.json)

    return node_examples

def get_node_edge_type(node_uid, edge_type):
    # node_id = '0x271c'
    # edge_type = 'friend'

    schema = get_schema()
    predicate_schema = [x for x in schema['schema'] if x['predicate'] == edge_type][0]
    # check schema to see if there is a lang attribute

    if predicate_schema.get('lang', False):
        query01 = """query
            {   all(func: uid(%s), first: 50)
                { %s@* @facets
                }
            }""" % (node_uid, edge_type)        
    else:
        query01 = """query
            {   all(func: uid(%s), first: 50)
                { %s @facets
                  { uid expand(_all_) { uid } }
                }
            }""" % (node_uid, edge_type)

    print(query01)

    res01 = client.txn(read_only=True).query(query01)
    edges = json.loads(res01.json)['all']

    if len(edges) > 0 and edge_type in edges[0]:
        edges = edges[0][edge_type]
    elif len(edges) > 0:
        edges = edges[0]
    elif len(edges) == 0:
        edges = [{}]

    return edges

